﻿//
// (C) 2013 Intel Corporation
//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Diagnostics;

namespace TBT_FW_Update_Tool
{
    class FwUpdateDllApi
    {
        private static IntPtr m_dllPointer = IntPtr.Zero;
        private const UInt32 DLL_LOAD_ERROR = 1;

        [DllImport("kernel32.dll")]
        static extern IntPtr LoadLibrary(string dllToLoad);

        [DllImport("kernel32.dll")]
        static extern IntPtr GetProcAddress(IntPtr hModule, string procedureName);


        [DllImport("kernel32.dll")]
        static extern bool FreeLibrary(IntPtr hModule);

        static public UInt32 LoadDll()
        {
            m_dllPointer = LoadLibrary(Environment.GetFolderPath(Environment.SpecialFolder.System) + "\\FwUpdateSdk.dll");
            if (m_dllPointer == IntPtr.Zero)
            {  
                return DLL_LOAD_ERROR;
            }
            return 0;
        }

        static public UInt32 UnloadDll()
        {
            if (FreeLibrary(m_dllPointer))
            {
                return 0;
            }
            return DLL_LOAD_ERROR;
        }
        
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate UInt32 FwUpdatePerformUpdateDelegate(UInt32 buffer_size, IntPtr buffer);

        static public UInt32 FwUpdate_PerformUpdate(UInt32 buffer_size, IntPtr buffer)
        {
            IntPtr pAddressOfFunctionToCall = GetProcAddress(m_dllPointer, "FwUpdatePerformUpdate");
            if (pAddressOfFunctionToCall == IntPtr.Zero)
            {
                return DLL_LOAD_ERROR;
            }
            try
            {
                FwUpdatePerformUpdateDelegate function = (FwUpdatePerformUpdateDelegate)Marshal.GetDelegateForFunctionPointer(pAddressOfFunctionToCall, typeof(FwUpdatePerformUpdateDelegate));
                if (function != null)
                {
                    return function(buffer_size, buffer);
                }
                return DLL_LOAD_ERROR;
            }
            catch (Exception)
            {
                return DLL_LOAD_ERROR;
            }
        }

        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate UInt32 FwUpdateCheckDriverConnectionDelegate();

        static public UInt32 FwUpdate_CheckDriverConnection()
        {
            IntPtr pAddressOfFunctionToCall = GetProcAddress(m_dllPointer, "FwUpdateCheckDriverConnection");
            if (pAddressOfFunctionToCall == IntPtr.Zero)
            {
                return DLL_LOAD_ERROR;
            }
                        
            try
            {
                FwUpdateCheckDriverConnectionDelegate function = (FwUpdateCheckDriverConnectionDelegate)Marshal.GetDelegateForFunctionPointer(pAddressOfFunctionToCall, typeof(FwUpdateCheckDriverConnectionDelegate));
                if (function != null)
                {
                    return function();
                }
                return DLL_LOAD_ERROR;
            }
            catch (Exception)
            {
                return DLL_LOAD_ERROR;
            }
            

        }


    }
}
