﻿//
// (C) 2013 Intel Corporation
//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Diagnostics;

namespace TBT_FW_Update_Tool
{
    public enum FwUpdateExecStatus {
        UpdateSuccess = 100,
        FileUnpackFailure,
        DllLoadFailure,
        NoDriverLoaded,
        ImageWriteFailure,
        AnotherError
    }

    class FwUpdateModule
    {
        public FwUpdateModule()
        { 
       
        }

        public FwUpdateExecStatus ExecuteUpdateFlow(byte[] image)
        {

            FwUpdateExecStatus tempStatus = CheckDriverConnection();
            if (tempStatus != FwUpdateExecStatus.UpdateSuccess)
            {
                return tempStatus;
            }
            else
            {
                FwUpdateExecStatus imageWriteStatus = FullImageWriteFlow(image);
                if (imageWriteStatus != FwUpdateExecStatus.UpdateSuccess)
                {
                    return imageWriteStatus;
                }
            }

            return FwUpdateExecStatus.UpdateSuccess;
        }

        private FwUpdateExecStatus FullImageWriteFlow(byte[] image)
        {
            uint libCoreStatus;

            unsafe
            {
                fixed (byte* p = image)
                {
                    IntPtr imagePtr = (IntPtr)p;
                    libCoreStatus = FwUpdateDllApi.FwUpdate_PerformUpdate((UInt32)image.Length, imagePtr);
                }
            }

            if (libCoreStatus != 0)
            {
                return FwUpdateExecStatus.ImageWriteFailure;
            }
           
            return FwUpdateExecStatus.UpdateSuccess;
        }


        private FwUpdateExecStatus CheckDriverConnection()
        {        
            int triesLeft = 3;
            UInt32 status;
            while( triesLeft > 0)
            {
                status = FwUpdateDllApi.FwUpdate_CheckDriverConnection();
                --triesLeft;

                if (status == 0) //driver is loaded
                {
                    return FwUpdateExecStatus.UpdateSuccess;
                }
                else //driver is not loaded
                {
                    if (triesLeft > 0)
                    {
                        DialogResult result = MessageBox.Show("There is no Thunderbolt\u2122 device connected to the system\n" +
                                                               "or Thunderbolt\u2122 Software is not present in the system.\n" +
                                                                "Make sure you have Thunderbolt\u2122 Software installed.\n" +
                                                                "Then please connect Thunderbolt\u2122 device and press Retry or press Cancel to abort Firmware update process",
                                                                "Thunderbolt\u2122 Firmware update utility",
                                                                MessageBoxButtons.RetryCancel,
                                                                MessageBoxIcon.Warning);

                        if (result == DialogResult.Cancel)
                        {
                            return FwUpdateExecStatus.NoDriverLoaded;
                        }
                    }
                   
                }

            }
            MessageBox.Show("The update process cannot be performed because there is no device connected to Thunderbolt\u2122 port or Thunderbolt\u2122 Software is not present in the system.\n Aborting...", "Thunderbolt\u2122 Firmware update utility", MessageBoxButtons.OK, MessageBoxIcon.Error);
            return FwUpdateExecStatus.NoDriverLoaded;

        }

    }
}
